@@ -36,6 +36,7 @@
 #include <cups/cups.h>
 #include <cups/ppd.h>
 #include <cups/raster.h>
+#include <cupsfilters/ipp.h>
 #include <cupsfilters/ppdgenerator.h>
 
 static int              debug = 0;
@@ -58,13 +59,18 @@ list_printers (int mode)
 		buffer[8192];		/* Copy buffer */
   cups_file_t	*fp;			/* Post-processing input file */
   char		*ptr;			/* Pointer into string */
-  char          *service_uri = NULL,
+  char          *scheme = NULL,
+                *service_name = NULL,
+                *domain = NULL,
+                *reg_type = NULL,
                 *txt_usb_mfg = NULL,
                 *txt_usb_mdl = NULL,
                 *txt_product = NULL,
                 *txt_ty = NULL,
                 *txt_pdl = NULL,
-		value[256],		/* Value string */
+                value[256],             /* Value string */
+                service_uri[2048],      /* URI to list for this service */
+                service_host_name[1024],/* "Host name" for assembling URI */
                 make_and_model[1024],	/* Manufacturer and model */
                 make[512],              /* Manufacturer */
 		model[256],		/* Model */
@@ -78,7 +84,7 @@ list_printers (int mode)
    * for our desired output.
    */
 
-  /* ippfind ! --txt printer-type --and \( --txt-pdl image/pwg-raster --or --txt-pdl image/urf \) -x echo -en '{service_uri}\t{txt_usb_MFG}\t{txt_usb_MDL}\t{txt_product}\t{txt_ty}\t{service_name}\t{txt_pdl}\n' \; */
+  /* ippfind ! --txt printer-type --and \( --txt-pdl image/pwg-raster --or --txt-pdl image/urf \) -x echo -en '{service_scheme}\t{service_name}\t{service_domain}\t{txt_usb_MFG}\t{txt_usb_MDL}\t{txt_product}\t{txt_ty}\t{service_name}\t{txt_pdl}\n' \; */
 
   i = 0;
   ippfind_argv[i++] = "ippfind";
@@ -110,9 +116,9 @@ list_printers (int mode)
   ippfind_argv[i++] = "echo";             /* Output the needed data fields */
   ippfind_argv[i++] = "-en";              /* separated by tab characters */
   if (mode > 0)
-    ippfind_argv[i++] = "{service_uri}\t{txt_usb_MFG}\t{txt_usb_MDL}\t{txt_product}\t{txt_ty}\t{txt_pdl}\n";
+    ippfind_argv[i++] = "{service_scheme}\t{service_name}\t{service_domain}\t{txt_usb_MFG}\t{txt_usb_MDL}\t{txt_product}\t{txt_ty}\t{txt_pdl}\n";
   else
-    ippfind_argv[i++] = "{service_uri}\n";
+    ippfind_argv[i++] = "{service_scheme}\t{service_name}\t{service_domain}\t\n";
   ippfind_argv[i++] = ";";
   ippfind_argv[i++] = NULL;
   /*for (i = 0; ippfind_argv[i]; i++) fprintf(stderr, "%s ", ippfind_argv[i]);
@@ -175,25 +181,50 @@ list_printers (int mode)
 
     while ((bytes = cupsFileGetLine(fp, buffer, sizeof(buffer))) > 0)
     {
-      if (strncasecmp(buffer, "ipp:", 4) && strncasecmp(buffer, "ipps:", 4)) {
-	/* Invalid IPP URI */
-	fprintf(stderr, "EROOR: Invalid IPP URI: %s\n", buffer);
-	continue;
-      }
+      /* Mark all the fields of the output of ippfind */
+      ptr = buffer;
+      /* First, build the DNS-SD-service-name-based URI ... */
+      while (ptr && !isalnum(*ptr & 255)) ptr ++;
+      if (!strncasecmp(ptr, "ipp", 3) && ptr[3] == '\t') {
+	scheme = ptr;
+	ptr += 3;
+	*ptr = '\0';
+	ptr ++;
+	reg_type = "_ipp._tcp";
+      } else if (!strncasecmp(ptr, "ipps", 4) && ptr[4] == '\t') {
+	scheme = ptr;
+	ptr += 4;
+	*ptr = '\0';
+	ptr ++;
+	reg_type = "_ipps._tcp";
+      } else
+	goto read_error;
+      service_name = ptr;
+      ptr = memchr(ptr, '\t', sizeof(buffer) - (ptr - buffer));
+      if (!ptr) goto read_error;
+      *ptr = '\0';
+      ptr ++;
+      domain = ptr;
+      ptr = memchr(ptr, '\t', sizeof(buffer) - (ptr - buffer));
+      if (!ptr) goto read_error;
+      *ptr = '\0';
+      ptr ++;
+      snprintf(service_host_name, sizeof(service_host_name) - 1, "%s.%s.%s",
+	       service_name, reg_type, domain);
+      httpAssembleURIf(HTTP_URI_CODING_ALL, service_uri,
+		       sizeof(service_uri) - 1,
+		       scheme, NULL,
+		       service_host_name, 0, "/");
+      /* ... second, complete the output line, either URI-only or with
+	 extra info for CUPS */
       if (mode == 0)
 	/* Manual call on the command line */
-	printf("%s", buffer);
+	printf("%s\n", service_uri);
       else {
 	/* Call by CUPS, either as PPD generator
 	   (/usr/lib/cups/driver/, with "list" command line argument)
 	   or as backend in discovery mode (/usr/lib/cups/backend/,
 	   env variable "SOFTWARE" starts with "CUPS") */
-	ptr = buffer;
-	service_uri = ptr;
-	ptr = memchr(ptr, '\t', sizeof(buffer) - (ptr - buffer));
-	if (!ptr) goto read_error;
-	*ptr = '\0';
-	ptr ++;
 	txt_usb_mfg = ptr;
 	ptr = memchr(ptr, '\t', sizeof(buffer) - (ptr - buffer));
 	if (!ptr) goto read_error;
@@ -223,10 +254,10 @@ list_printers (int mode)
 	make[0] = '\0';
 	pdl[0] = '\0';
 	device_id[0] = '\0';
-	strncpy(model, "Unknown", sizeof(model));
+	strncpy(model, "Unknown", sizeof(model) - 1);
 	
 	if (txt_usb_mfg[0] != '\0') {
-	  strncpy(make, txt_usb_mfg, sizeof(make));
+	  strncpy(make, txt_usb_mfg, sizeof(make) - 1);
 	  if (strlen(txt_usb_mfg) > 511)
 	    make[511] = '\0';
 	  ptr = device_id + strlen(device_id);
@@ -234,7 +265,7 @@ list_printers (int mode)
 		   "MFG:%s;", txt_usb_mfg);
 	}
 	if (txt_usb_mdl[0] != '\0') {
-	  strncpy(model, txt_usb_mdl, sizeof(model));
+	  strncpy(model, txt_usb_mdl, sizeof(model) - 1);
 	  if (strlen(txt_usb_mdl) > 255)
 	    model[255] = '\0';
 	  ptr = device_id + strlen(device_id);
@@ -246,20 +277,20 @@ list_printers (int mode)
 	    if ((ptr = txt_product + strlen(txt_product) - 1) > txt_product &&
 		*ptr == ')')
 	      *ptr = '\0';
-	    strncpy(model, txt_product + 1, sizeof(model));
+	    strncpy(model, txt_product + 1, sizeof(model) - 1);
 	    if ((strlen(txt_product) + 1) > 255)
 	      model[255] = '\0';
 	  } else
-	    strncpy(model, txt_product, sizeof(model));
+	    strncpy(model, txt_product, sizeof(model) - 1);
 	} else if (txt_ty[0] != '\0') {
-	  strncpy(model, txt_ty, sizeof(model));
+	  strncpy(model, txt_ty, sizeof(model) - 1);
 	  if (strlen(txt_ty) > 255)
 	    model[255] = '\0';
 	  if ((ptr = strchr(model, ',')) != NULL)
 	    *ptr = '\0';
 	}
 	if (txt_pdl[0] != '\0') {
-	  strncpy(pdl, txt_pdl, sizeof(pdl));
+	  strncpy(pdl, txt_pdl, sizeof(pdl) - 1);
 	  if (strlen(txt_pdl) > 255)
 	    pdl[255] = '\0';
 	}
@@ -329,7 +360,7 @@ list_printers (int mode)
 	  snprintf(make_and_model, sizeof(make_and_model), "%s %s",
 		   make, model);
 	else
-	  strncpy(make_and_model, model, sizeof(make_and_model));
+	  strncpy(make_and_model, model, sizeof(make_and_model) - 1);
 
 	if (mode == 1)
 	  /* Call with "list" argument (PPD generator in list mode */
@@ -470,61 +501,29 @@ list_printers (int mode)
 int
 generate_ppd (const char *uri)
 {
-  int uri_status, host_port;
-  http_t *http = NULL;
-  char scheme[10], userpass[1024], host_name[1024], resource[1024];
-  ipp_t *request, *response = NULL;
-  ipp_attribute_t *attr;
+  ipp_t *response = NULL;
   char buffer[65536], ppdname[1024];
-  int i, fd, bytes;
-  static const char * const pattrs[] =
-  {
-    "all",
-    "media-col-database"
-  };
+  int fd, bytes;
+  char *ptr1, *ptr2;
 
   /* Request printer properties via IPP to generate a PPD file for the
-     printer (mainly IPP Everywhere printers)
-     If we work with Systen V interface scripts use this info to set
-     option defaults. */
-  uri_status = httpSeparateURI(HTTP_URI_CODING_ALL, uri,
-			       scheme, sizeof(scheme),
-			       userpass, sizeof(userpass),
-			       host_name, sizeof(host_name),
-			       &(host_port),
-			       resource, sizeof(resource));
-  if (uri_status != HTTP_URI_OK) {
-    fprintf(stderr, "ERROR: Invalid URI: %s\n", uri);
-    goto fail;
-  }
-  if ((http = httpConnect2(host_name, host_port, NULL, AF_UNSPEC,
-			   HTTP_ENCRYPT_IF_REQUESTED, 1, 5000, NULL)) ==
-      NULL) {
-    fprintf(stderr, "ERROR: Cannot connect to remote printer %s (%s:%d)\n",
-	    uri, host_name, host_port);
-    goto fail;
-  }
-  request = ippNewRequest(IPP_OP_GET_PRINTER_ATTRIBUTES);
-  ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_URI, "printer-uri",
-	       NULL, uri);
-  ippAddStrings(request, IPP_TAG_OPERATION, IPP_TAG_KEYWORD,
-		"requested-attributes", sizeof(pattrs) / sizeof(pattrs[0]),
-		NULL, pattrs);
-  response = cupsDoRequest(http, request, resource);
-
-  /* Log all printer attributes for debugging */
+     printer */
+  response = get_printer_attributes(uri, NULL, 0, NULL, 0, 1);
   if (debug) {
-    attr = ippFirstAttribute(response);
-    while (attr) {
-      fprintf(stderr, "DEBUG2: Attr: %s\n", ippGetName(attr));
-      ippAttributeString(attr, buffer, sizeof(buffer));
-      fprintf(stderr, "DEBUG2: Value: %s\n", buffer);
-      for (i = 0; i < ippGetCount(attr); i ++)
-	fprintf(stderr, "DEBUG2: Keyword: %s\n",
-		ippGetString(attr, i, NULL));
-      attr = ippNextAttribute(response);
+    ptr1 = get_printer_attributes_log;
+    while(ptr1) {
+      ptr2 = strchr(ptr1, '\n');
+      if (ptr2) *ptr2 = '\0';
+      fprintf(stderr, "DEBUG2: %s\n", ptr1);
+      if (ptr2) *ptr2 = '\n';
+      ptr1 = ptr2 ? (ptr2 + 1) : NULL;
     }
   }
+  if (response == NULL) {
+    fprintf(stderr, "ERROR: Unable to create PPD file: Could not poll sufficient capability info from the printer (%s, %s) via IPP!\n",
+	    uri, resolve_uri(uri));
+    goto fail;
+  }
 
   /* Generate the PPD file */
   if (!ppdCreateFromIPP(ppdname, sizeof(ppdname), response, NULL, NULL, 0, 0)) {
@@ -543,7 +542,6 @@ generate_ppd (const char *uri)
   }
 
   ippDelete(response);
-  httpClose(http);
 
   /* Output of PPD file to stdout */
   fd = open(ppdname, O_RDONLY);
@@ -555,9 +553,8 @@ generate_ppd (const char *uri)
   return 0;
   
  fail:
-  ippDelete(response);
-  if (http)
-    httpClose(http);
+  if (response)
+    ippDelete(response);
 
   return 1;
 }
